"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClientService = void 0;
const BaseAPI_1 = require("./BaseAPI");
const Error_1 = require("./Error");
var Operation;
(function (Operation) {
    Operation["QUERY"] = "query";
    Operation["MUTATION"] = "mutation";
})(Operation || (Operation = {}));
class ClientService extends BaseAPI_1.BaseAPI {
    constructor(client, baseUrl, config) {
        super(client, baseUrl, config);
    }
    async getTicketById(args, body) {
        try {
            const query = this.generateOperation(Operation.QUERY, 'Ticket', args, body);
            const { data } = await this.post(query);
            this.errorHandler(data);
            return data.data.Ticket;
        }
        catch (e) {
            return Promise.reject(e);
        }
    }
    async getTickets(args, body) {
        try {
            const query = this.generateOperation(Operation.QUERY, 'Tickets', args, body);
            const { data } = await this.post(query);
            this.errorHandler(data);
            return data.data.Tickets;
        }
        catch (e) {
            return Promise.reject(e);
        }
    }
    async suggestAnswer(args, body) {
        try {
            const query = this.generateOperation(Operation.QUERY, 'SuggestAnswer', args, body);
            const { data } = await this.post(query);
            this.errorHandler(data);
            // this.query()
            return data.data.SuggestAnswer;
        }
        catch (e) {
            return Promise.reject(e);
        }
    }
    async getTicketDepartments(body) {
        try {
            const query = this.generateOperation(Operation.QUERY, 'TicketDepartments', {}, body);
            const { data } = await this.post(query);
            this.errorHandler(data);
            return data.data.TicketDepartments;
        }
        catch (e) {
            return Promise.reject(e);
        }
    }
    createFormDate(mutation, attachments) {
        const bodyFormData = new FormData();
        bodyFormData.set('operations', JSON.stringify({
            query: `${mutation}`,
            variables: { attachments }
        }));
        bodyFormData.set('operationName', '');
        bodyFormData.set('map', JSON.stringify({ "file": ["variables.file"] }));
        bodyFormData.append('file', attachments);
        return bodyFormData;
    }
    async ticketAdd(args, body) {
        try {
            const mutation = this.generateOperation(Operation.MUTATION, 'TicketAdd', args, body);
            const { attachments } = args;
            const queryData = attachments ? this.createFormDate(mutation, attachments) : mutation;
            // console.log('@@@@@')
            // console.log(mutation)
            // return 
            const { data } = await this.post(queryData);
            this.errorHandler(data);
            return data.data.TicketAdd;
        }
        catch (e) {
            return Promise.reject(e);
        }
    }
    async ticketAddReply(args, body) {
        try {
            const mutation = this.generateOperation(Operation.MUTATION, 'TicketAddReply', args, body);
            const { attachments } = args;
            const queryData = attachments ? this.createFormDate(mutation, attachments) : mutation;
            const { data } = await this.post(queryData);
            this.errorHandler(data);
            return data.data.TicketAddReply;
        }
        catch (e) {
            return Promise.reject(e);
        }
    }
    async ticketFeedback(args) {
        try {
            const mutation = this.generateOperation(Operation.MUTATION, 'TicketFeedback', args, []);
            const { data } = await this.post(mutation);
            this.errorHandler(data);
            return data.data.TicketFeedback;
        }
        catch (e) {
            return Promise.reject(e);
        }
    }
    async ticketReplyFeedback(args) {
        try {
            const mutation = this.generateOperation(Operation.MUTATION, 'TicketReplyFeedback', args, []);
            const { data } = await this.post(mutation);
            this.errorHandler(data);
            return data.data.TicketReplyFeedback;
        }
        catch (e) {
            return Promise.reject(e);
        }
    }
    async ticketClose(args) {
        try {
            const mutation = this.generateOperation(Operation.MUTATION, 'TicketClose', args, []);
            const { data } = await this.post(mutation);
            this.errorHandler(data);
            return data.data.TicketClose;
        }
        catch (e) {
            return Promise.reject(e);
        }
    }
    generateArgs(args) {
        const enums = ["LOW", "MEDIUM", "HIGH", "HAPPY", "NEUTRAL", "SAD", "NONE", "NEW", "IN_PROGRESS", "SUPPORT_RESPONSE", "CLOSED"];
        const isEnums = (x) => enums.includes(x);
        let str = ``;
        for (let arg in args) {
            const value = args[arg];
            if (typeof value === "string") {
                str += `${arg}:${isEnums(value) ? value : `"${value}"`},`;
            }
            else {
                str += `${arg}:${value},`;
            }
        }
        return str;
    }
    generateOperation(type, name, args, body) {
        let argument = args;
        if (this.hasAttachment(args)) {
            const { attachments } = args, restArgs = __rest(args, ["attachments"]);
            argument = restArgs;
        }
        const generatedType = this.hasAttachment(args) ? `${type} ${name}($file: Upload!)` : `${type}`;
        const generatedArgs = `${name} ${Object.keys(args).length ? `(${this.generateArgs(argument)} ${this.hasAttachment(args) ? 'attachments: $file' : ''} )` : ''}`;
        const generatedBody = `${body.length ? `{ ${this.generateBody(body)} }` : this.generateBody(body)}`;
        return `${generatedType} { ${generatedArgs} ${generatedBody} }`;
    }
    hasAttachment(args) {
        return args.hasOwnProperty('attachments');
    }
    generateBody(body) {
        let str = '';
        for (let item of body) {
            if (typeof (item) === 'object') {
                for (let key in item) {
                    str += `${key} { ${this.generateBody(item[key])} }`;
                }
            }
            else {
                str += ` ${item} `;
            }
        }
        return str;
    }
    errorHandler(data) {
        var _a;
        if (data.errors) {
            const { errors } = data;
            if ((_a = errors[0].validation) === null || _a === void 0 ? void 0 : _a.length) {
                throw new Error_1.ValidationError(errors[0].message, errors);
            }
            else {
                throw new Error_1.ApplicationError(errors[0].message, errors);
            }
        }
    }
}
exports.ClientService = ClientService;
