export type Maybe<T> = T | null

// TODO: change HttpClient
export interface Client {
    post(url?: string, data?: any, config?: ClientConfig): Promise<any>;
}

export interface ClientConfig {
    headers?: { [key: string]: string | boolean | number; };
    baseUrl?: string;
}

export type Scalars = {
    ID: string | number
    String: string
    Boolean: boolean
    Int: number
    Float: number
    DateTime: any
}

export type Query = {
    Ticket: Ticket
    Tickets: TicketSimplePagination
    TicketDepartments: Array<TicketDepartment>
    SuggestAnswer: Array<SuggestAnswer>
}


export type Mutation = {
    TicketAdd: Ticket
    TicketAddReply: TicketReply
    TicketFeedback: Scalars['Boolean']
    TicketReplyFeedback: Scalars['Boolean']
    TicketClose: Scalars['Boolean']
}

export type ArgsQuery = {
    Ticket: QueryTicketArgs
    Tickets: QueryTicketsArgs
    SuggestAnswer: QuerySuggestAnswerArgs
}

export type ArgsMutation = {
    TicketAdd: MutationTicketAddArgs
    TicketAddReply: MutationTicketAddReply
    TicketFeedback: MutationTicketFeedbackArgs
    TicketReplyFeedback: MutationTicketReplyFeedbackArgs
    TicketClose: MutationTicketCloseArgs
}

export type Ticket = {
    id: Scalars['ID']
    user_id: Scalars['ID']
    user?: Maybe<User>
    assigned_user?: User
    user_seen_at?: Scalars['String']
    second_user?: User
    second_user_seen_at?: Scalars['String']
    title: Scalars['String']
    priority?: TicketPriorityEnum
    flag?: Scalars['String']
    feedback_type?: TicketFeedbackTypeEnum
    created_at?: Scalars['String']
    updated_at?: Scalars['String']
    ticketDepartment?: TicketDepartment
    ticketReplies?: Array<TicketReply>
    status?: TicketStatusEnum
}

export type TicketSimplePagination = {
    data?: TicketSimple[]
    total: Scalars['Int']
    per_page: Scalars['Int']
    current_page: Scalars['Int']
    from?: Scalars['Int']
    to?: Scalars['Int']
    last_page: Scalars['Int']
    has_more_pages: Scalars['Boolean']
}

export type TicketDepartment = {
    id: Scalars['ID']
    title?: Scalars['String']
    user_access: Scalars['Boolean']
}

export type SuggestAnswer = {
    title?: Scalars['String']
    message?: Scalars['String']
    url?: Scalars['String']
}

export type TicketReply = {
    user: User
    content?: Scalars['String']
    feedback_type: TicketReplyFeedbackTypeEnum
    feedback: Feedback
    create_at: Scalars['String']
    updated_at: Scalars['String']
    user_id: Scalars['ID']
    attachments: Array<Attachment>
}

export type User = {
    id: Scalars['ID']
    avatar?: Scalars['String']
    full_name?: Scalars['String']
    team_member?: Scalars['Boolean']
}

export enum TicketPriorityEnum {
    LOW = 'LOW',
    MEDIUM = 'MEDIUM',
    HIGH = 'HIGH',
}

export enum TicketFeedbackTypeEnum {
    HAPPY = 'HAPPY',
    NEUTRAL = 'NEUTRAL',
    SAD = 'SAD',
    NONE = 'NONE'
}

export enum TicketStatusEnum {
    NEW = 'NEW',
    IN_PROGRESS = 'IN_PROGRESS',
    SUPPORT_RESPONSE = 'SUPPORT_RESPONSE',
    CLOSED = 'CLOSED',
}

export type TicketSimple = {
    id: Scalars['ID']
    user_id: Scalars['ID']
    user?: User
    assigned_user?: Maybe<User>
    user_seen_at?: Maybe<Scalars['String']>
    second_user?: Maybe<User>
    second_user_seen_at?: Maybe<Scalars['String']>
    title: Scalars['String']
    priority?: Maybe<TicketPriorityEnum>
    flag?: Maybe<Scalars['String']>
    feedback_type?: Maybe<TicketFeedbackTypeEnum>
    status?: Maybe<TicketStatusEnum>
    created_at?: Maybe<Scalars['String']>
    ticketDepartment?: Maybe<TicketDepartment>
}

export enum TicketReplyFeedbackTypeEnum {
    INSUFFICIENT = 'INSUFFICIENT',
    DISRESPECTFUL = 'DISRESPECTFUL',
    NONE = 'NONE'
}

export type Feedback = {
    feedback_type: FeedbackTypeEnum
}

export type Attachment = {
    url?: Scalars['String']
    extension?: Scalars['String']
}

export enum FeedbackTypeEnum {
    INSUFFICIENT = 'INSUFFICIENT',
    DISRESPECTFUL = 'DISRESPECTFUL',
    NONE = 'NONE'
}

export type QueryTicketArgs = {
    id: Scalars['ID']
}

export type QueryTicketsArgs = {
    id?: Maybe<Scalars['ID']>
    limit?: Maybe<Scalars['Int']>
    page?: Maybe<Scalars['Int']>
}

export type QuerySuggestAnswerArgs = {
    title: Scalars['String']
}

export type MutationTicketAddArgs = {
    ticket_department_id: Scalars['ID']
    title: Scalars['String']
    priority?: Maybe<TicketPriorityEnum> | string
    content: Scalars['String']
    attachments?: any
}

export type MutationTicketAddReply = {
    ticket_id: Scalars['ID']
    content: Scalars['String']
    attachments?: any
}

export type MutationTicketFeedbackArgs = {
    ticket_id: Scalars['ID']
    feedback_type?: Maybe<TicketFeedbackTypeEnum>
}

export type MutationTicketReplyFeedbackArgs = {
    ticket_reply_id: Scalars['ID']
    feedback_type?: Maybe<TicketReplyFeedbackTypeEnum>
}

export type MutationTicketCloseArgs = {
    ticket_id: Scalars['String']
}

